''' Raspberry Pi Pico - Voltmeter
 Needs the appropriate voltmeter code on a pico connected to /dev/serial0
 Sends via uart
 Works with ADC pins 0 to 4
 pins 0 to 2 are gp26 to gp28
 pin 3 unused
 pin 4 is internal temp sensor
 See: www.penguintutor.com/projects/pico '''


import serial
import time
from math import floor

WIDTH=800
HEIGHT=600

ser = serial.Serial('/dev/serial0', 115200, timeout=5)

pico_img = Actor('pico')
pico_img.topleft = 30, 160

selected_pin = 0
voltage = 0

pin_img_disabled = "pin-disabled"
pin_img_enabled = "pin-enabled"

pin_imgs = [
    Actor(pin_img_enabled, (370, 360)),
    Actor(pin_img_disabled, (370, 335)),
    Actor(pin_img_disabled, (370, 305)),
    Actor(pin_img_disabled, (-20, -20)),        # GP29 not currently used - display offscreen
    Actor(pin_img_disabled, (370, 140))
    ]

# How long to delay between checks (approx 60th of seconds)
delay_check = 30
last_checked = 0

def draw ():
    screen.clear()
    screen.fill((255,255,255))
    pico_img.draw()
    for this_pin in pin_imgs :
        this_pin.draw()
    # Title
    screen.draw.text("Voltmeter", centery=70, centerx=400, fontsize=60, color="#F00095")
    volt_int = floor(voltage)
    volt_fraction = floor((voltage-volt_int)*100)
    screen.draw.text(str(volt_int), topright=(545,300), fontname="segment7standard", fontsize=94, color="#00FF00")
    screen.draw.text("."+str(volt_fraction), topleft=(500,300), fontname="segment7standard", fontsize=94, color="#00FF00")
    screen.draw.text("volts", topleft=(660,360), color="#000000")
    screen.draw.text("Temp", topleft=(306,130), color="#000000")



def update ():
    global voltage, last_checked

    # delay how often we update
    if (last_checked < delay_check) :
        last_checked += 1
        return
    last_checked = 0

    #Convert pin to character (add 48 = '0') and convert to byte
    #Send to serial
    ser.write((selected_pin+48).to_bytes(1, byteorder='little'))

    # read response back from Pico
    input = ser.read_until()
    sub_input = input.decode("utf-8").split()
    if (str(sub_input[1]) == str(selected_pin)):
        # save as voltage
        voltage = float(sub_input[3])
    # Uncomment lines below for debugging serial connection
    #print (str(sub_input))
    #print ("Voltage : " + str(voltage))

def on_mouse_down(pos, button):
    global selected_pin
    if button == mouse.LEFT:
       # Check each of the buttons
        for pin_id in range (0,len(pin_imgs)) :
            if pin_imgs[pin_id].collidepoint(pos):
                # reset the previous pin
                pin_imgs[selected_pin].image = pin_img_disabled
                # set this to the new pin
                selected_pin = pin_id
                pin_imgs[selected_pin].image = pin_img_enabled